#include "doorkeeper.hpp"
#include "doorkeepertestConfig.h"
#include "platform.h"
#include "platformstrings.h"
#include "helpers/asciimapsource.hpp"
#include <iostream>
#include <fstream>
#include <string>
#include <memory>

template <typename Device, typename Tile>
struct LayerWithData {
	std::unique_ptr<dk::TileMapData<Tile, 2>> tilemap;
	std::unique_ptr<Device> device;
	dk::Layer<Tile, 2>* layer;
	std::string path;
};

namespace {
	template <typename Device, typename Tile>
	void createLayer ( dk::Tyler<2>& parTiler, LayerWithData<Device, Tile>& parOut );
	void printWelcome ( void );
} //unnamed namespace

int main() {
	typedef dk::Tyler<2>::coords coords2;
	using dk::TileMapData;
	using dkh::AsciiMapSource;

	printWelcome();

	dk::Tyler<2> tiler(coords2(10, 6), coords2(64));

	LayerWithData<AsciiMapSource, int> bottomLayer;
	bottomLayer.path = DATA_PATH"/test.map";
	bottomLayer.device = std::unique_ptr<AsciiMapSource>(new AsciiMapSource(bottomLayer.path));
	bottomLayer.tilemap = std::unique_ptr<TileMapData<int, 2>>(new TileMapData<int, 2>(*bottomLayer.device));
	bottomLayer.layer = &tiler.push_layer(*bottomLayer.tilemap);

	LayerWithData<AsciiMapSource, int> topLayer;
	topLayer.path = DATA_PATH"/test_2.map";
	topLayer.device = std::unique_ptr<AsciiMapSource>(new AsciiMapSource(topLayer.path));
	topLayer.tilemap = std::unique_ptr<TileMapData<int, 2>>(new TileMapData<int, 2>(*topLayer.device));
	topLayer.layer = &tiler.push_layer(*topLayer.tilemap);

	dk::Viewport<2> viewport(tiler, coords2(10, 6), coords2(0));
	{
		int col = 0;
		const auto tilecount = bottomLayer.layer->count();
		for (auto tile : *bottomLayer.layer) {
			std::cout << tile;
			++col;
			if (col == tilecount.x()) {
				col = 0;
				std::cout << '\n';
			}
		}
	}
	dk::Viewport<2> viewport_small(tiler, coords2(4, 4), coords2(0));
	{
		int col = 0;
		const auto tilecount = viewport_small.count();
		for (auto itTile = viewport_small.begin(*bottomLayer.layer), itTileEND = viewport_small.end(*bottomLayer.layer); itTile != itTileEND; ++itTile) {
			std::cout << *itTile;
			++col;
			if (col == tilecount.x()) {
				col = 0;
				std::cout << '\n';
			}
		}
	}

#if !defined(NDEBUG)
	std::cout << "Map size: " << tiler.map_size() << '\n';
#endif
	std::cout << "Total tiles: " << tiler.tiles_count() << '\n';
	return 0;
}

namespace {
	template <typename Device, typename Tile>
	void createLayer (dk::Tyler<2>& parTiler, LayerWithData<Device, Tile>& parOut) {
	}

	void printWelcome() {
		std::cout << "Welcome to " << APP_NAME << ' ' << DK_DEVICE_STRING << " version for " << DK_OS_STRING << ' ' << DK_ARCH_STRING << ' ' << DK_BIT_STRING;
#if defined(DK_ARM)
		std::cout << ' ' << DK_ARM_FAMILY_STRING << ' ' << DK_ARM_ARCH_STRING;
#endif
		std::cout << '\n';
	}
} //unnamed namespace
